Package: eiR
Type: Package
Title: Accelerated similarity searching of small molecules
Version: 1.52.0
Date: 2025-02-11
Author: Kevin Horan, Yiqun Cao and Tyler Backman
Maintainer: Thomas Girke <thomas.girke@ucr.edu>
Suggests: BiocStyle, knitcitations, knitr,
        knitrBootstrap,rmarkdown,RSQLite,codetools
Description: The eiR package provides utilities for accelerated
        structure similarity searching of very large small molecule
        data sets using an embedding and indexing approach.
License: Artistic-2.0
Depends: R (>= 2.10.0), ChemmineR (>= 2.15.15), methods, DBI
Imports: snow, tools, snowfall, RUnit, methods, ChemmineR, RCurl,
        digest, BiocGenerics, RcppAnnoy (>= 0.0.9)
biocViews: Cheminformatics, BiomedicalInformatics, Pharmacogenetics,
        Pharmacogenomics, MicrotitrePlateAssay, CellBasedAssays,
        Visualization, Infrastructure, DataImport, Clustering,
        Proteomics, Metabolomics
URL: https://github.com/girke-lab/eiR
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/eiR
git_branch: RELEASE_3_23
git_last_commit: 084dacc
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 21:54:16 UTC; biocbuild
