The eiR package provides utilities for accelerated structure similarity searching of very large small molecule 
data sets using an embedding and indexing approach.

eiR is hosted on [Bioconductor](http://www.bioconductor.org) where you can find the 
[Release](http://www.bioconductor.org/packages/2.14/bioc/html/eiR.html) version and the 
[Development](http://www.bioconductor.org/packages/devel/bioc/html/eiR.html) version.
